#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>	// perror()
#include <unistd.h>	// close()

#include "paradlib.h"
//#include "ppdev_paradlib.h"

extern "C" {
#include <linux/ppdev.h>
#include <linux/parport.h>
}

#define IOW_PIN		PARPORT_CONTROL_INIT
#define _IOR_PIN 	PARPORT_CONTROL_AUTOFD

#define _SA0_PIN	PARPORT_CONTROL_STROBE
#define _SA1_PIN	PARPORT_CONTROL_SELECT

const unsigned char CParadlib::op_table[9] =
	{0x00, 0x01, 0x02, 0x08, 0x09, 0x0a, 0x10, 0x11, 0x12};

CParadlib::CParadlib(char *ppdev_dev)
{
	dev = ppdev_dev;
	init_res = 0;
	ppdev_fd = -1;
}

void CParadlib::write(int reg, int val)
{
	writeReg(reg, val);
}

int CParadlib::isInit()
{
	return init_res;
}

void CParadlib::reset_card()
{
	int i;
	for (i=0; i<9; i++) {
		writeReg(0xb0 + i, 0); // keyoff
		writeReg(0x80 + op_table[i], 0xff); //fastest release
	}
	writeReg(0xbd, 0); // clear misc reg
}

void CParadlib::init()
{
	int res;
	unsigned int modes=0;
	unsigned char ctrl;

	if (ppdev_fd != -1) {
		fprintf(stderr, "Already init!\n");
		return;
	}
	
	ppdev_fd = open(dev, O_RDWR);
	if (ppdev_fd < 0) {
		perror("open");
		init_res = 1;
		return;
	}

	res = ioctl(ppdev_fd, PPEXCL);
	if (res<0) {
		perror("PPEXCL");
		close(ppdev_fd);
		init_res = 1;
		return;
	}

	res = ioctl(ppdev_fd, PPCLAIM);
	if (res<0) {
		perror("PPCLAIM");
		close(ppdev_fd);
		init_res = 1;
		return;
	}

	modes = IEEE1284_MODE_COMPAT;
	if (ioctl(ppdev_fd, PPNEGOT, &modes)) {
		perror("PPNEGOT");
		close(ppdev_fd);
		init_res = 1;
		return;
	}

	res = ioctl(ppdev_fd, PPGETMODES, &modes);
	if (res<0) {
		perror("Could not get availables modes\n");
		close (ppdev_fd);
		init_res = -1;
		return;
	}

//	printf("Modes: %08x :", modes);
	if (modes & PARPORT_MODE_PCSPP) { printf("SPP "); }
	if (modes & PARPORT_MODE_TRISTATE) { printf("TRISTATE "); }
	if (modes & PARPORT_MODE_EPP) { printf("EPP "); }
	if (modes & PARPORT_MODE_ECP) { printf("ECP "); }
	if (modes & PARPORT_MODE_COMPAT) { printf("COMPAT "); }
	if (modes & PARPORT_MODE_DMA) { printf("DMA "); }
//	printf("\n");	

	if (!(modes & PARPORT_MODE_PCSPP)) {
		fprintf(stderr, "Port does not support standard parallel port mode\n");
		init_res = -1;
		return;
	}
	if (!(modes & PARPORT_MODE_TRISTATE)) {
		fprintf(stderr, "Port does not support bidirectional mode\n");
		init_res = -1;
		return;
	}	
	printf("OK\n");

	ioctl(ppdev_fd, PPRCONTROL, &ctrl);
	// read high
	ctrl &= ~(_IOR_PIN);
	// Write high, sa0 low, sa1 low
	ctrl |= IOW_PIN | _SA0_PIN | _SA1_PIN;
	ioctl(ppdev_fd, PPWCONTROL, &ctrl);

	init_res = 0;

	//reset_card();
	
	return;
}

void CParadlib::writeReg(unsigned char reg, unsigned char data)
{
//	printf("w%02X -> %02X\n", reg, data);
	llwrite(0, reg, 6);
	llwrite(1, data, 35);
}

void CParadlib::llwrite(unsigned char lsb, unsigned char data, int num_loops)
{
	unsigned char ctrl;
	int i;
	
	if (ioctl(ppdev_fd, PPRCONTROL, &ctrl)) {
		perror("PPRCONTROL");
	}

	if (ioctl(ppdev_fd, PPWDATA, &data)) {
		perror("PPWDATA");
	}
	
	if (lsb) {
		ctrl &= ~_SA0_PIN;
	} else {
		ctrl |= _SA0_PIN;
	}

	ioctl(ppdev_fd, PPWCONTROL, &ctrl);	
	ctrl &= ~IOW_PIN;
	
	ioctl(ppdev_fd, PPWCONTROL, &ctrl);
	
	ctrl |= IOW_PIN;
	
	ioctl(ppdev_fd, PPWCONTROL, &ctrl);

	for (i=0; i<num_loops; i++)
		ioctl(ppdev_fd, PPWCONTROL, &ctrl);
	
}


